@@ -21,13 +21,13 @@
 # Encapsulates some image store structure and convetions.
 # 
 
-import os, shutil, md5, time, re
+import os, shutil, time, re
 import urllib,urllib2, urlparse
 
 import convirt.core.utils.constants
 from convirt.core.utils.constants import *
 from convirt.core.utils.utils import XMConfig, PyConfig, copyToRemote
-from convirt.core.utils.utils import read_python_conf, copyToRemote, mkdir2, get_path
+from convirt.core.utils.utils import read_python_conf, copyToRemote, mkdir2, get_path, md5_constructor
 
 import traceback
 
@@ -198,7 +198,7 @@ class ImageStore:
     # generate a unique hash
     @classmethod
     def getId(cls, name):
-        x = md5.new(name)
+        x = md5_constructor(name)
         x.update(str(time.time()))
         return x.hexdigest()
 
@@ -344,6 +344,10 @@ class ImageStore:
         if self._store_location is None or self._store_location is '':
             self._store_location = self.DEFAULT_STORE_LOCATION
 
+        if self._store_location: 
+            if self._store_location.find(' ') > -1:
+                raise Exception("Image store location can not contain spaces.")
+
         images_info = self._config.get(XMConfig.APP_DATA,
                                   prop_images)
         
@@ -649,7 +653,7 @@ def get_template_location():
     if path:
         return tfile
     else:
-        msg=  "ERROR: Couldn't find appliance_template %s!" % template
+        msg=  "ERROR: Couldn't find appliance_template. This is mostly installation problem."
         print msg
         raise Exception(msg)
 
@@ -714,10 +718,19 @@ class ImageUtils:
                                 progress.update(progress.START_PULSE,
                                                 "Copying " + src +
                                                 " to \n" + dest )  
-                            (out, code) = local.node_proxy.exec_cmd("cp -a " + \
-                                                                    src + \
-                                                                    " " + \
-                                                                    dest)
+                            if src.find("/dev/") == 0 or \
+                                    dest.find("/dev/") == 0:
+                                
+                                (out, code) = local.node_proxy.exec_cmd("dd if=" + \
+                                                                      src + \
+                                                                      " of=" + \
+                                                                       dest)
+                            else:
+
+                                (out, code) = local.node_proxy.exec_cmd("cp -a " + \
+                                                                      src + \
+                                                                      " " + \
+                                                                      dest)
                             if code != 0:
                                 raise Exception(out)
 
