@@ -23,7 +23,8 @@
 # machines/host/nodes with virtualization technology.
 
 from convirt.core.utils.NodeProxy import Node, NodeWrapper
-from convirt.core.utils.utils import XMConfig, LVMProxy
+from convirt.core.utils.utils import XMConfig, LVMProxy, PyConfig
+from convirt.core.utils.IPy import *
 from convirt.core.utils.constants import *
 from convirt.core.utils.phelper import HostValidationException,\
      AuthenticationException, CommunicationException
@@ -167,34 +168,19 @@ class ManagedNode:
         pass
 
     def get_os_info(self):
-      if len(self.environ['os_info']) <= 0:
-          os_values = self.node_proxy.exec_cmd( \
-                    'uname -r;uname -s; uname -m;'\
-                    )[0].split('\n')[:-1]
-          i = 0
-          for name in [key_os_release,key_os_system,key_os_machine]:
-              try:
-                  self.environ['os_info'][name] = os_values[i]
-              except ValueError:
-                  pass
-              i = i+1
       return self.environ['os_info']
     
     def get_network_info(self):
-      raw_network_list = self.environ['network_info']
-      network_list = []
-      if raw_network_list[0].find('not found') == -1:
-        for i in range(len(raw_network_list)):
-          split_item = raw_network_list[i].split("UP")
-          if cmp('',split_item[0]) !=0 :
-            split_item = split_item[0].split('\n')
-            interface_name = split_item[0].split()[0]
-            ip_addr = ''
-            if cmp('',split_item[1]) !=0:
-              for i in range(1, len(split_item)):
-                ip_addr += split_item[i] + '\n'
-            network_list.append(dict([(key_network_interface_name, interface_name), (key_network_ip, ip_addr)]))
-      return network_list
+        return self.environ['network_info']
+
+    def get_nic_info(self):
+        return self.environ['nic_info']
+
+    def get_bridge_info(self):
+        return self.environ['bridge_info']
+
+    def get_default_bridge(self):
+        return self.environ['default_bridge']
 
     def get_cpu_info(self):
       return self.environ['cpu_info']
@@ -206,7 +192,7 @@ class ManagedNode:
       return self.environ['memory_info']
 
     def get_os_info_display_names(self):
-      display_dict = {key_os_release:display_os_release,key_os_system:display_os_system,key_os_machine:display_os_machine}
+      display_dict = {key_os_release:display_os_release,key_os_machine:display_os_machine,key_os_distro_string:display_os_distro}
       return display_dict
 
     def get_network_info_display_names(self):
@@ -326,6 +312,20 @@ class NodeEnvHelper:
         self.__dict = None
         self.__populateDictionary()
 
+    # try to get the network definition from ip and netmask
+    def populate_network(self, n):
+        if n.get("ip") and n.get("netmask"):
+            try:
+                ip = IP(n.get("ip"))
+                mask = IP(n.get("netmask"))
+                net_int = ip.int() & mask.int()
+                net_ip = IP(net_int)
+                net_ip_mask = IP(net_ip.strNormal() + "/" + mask.strNormal())
+                net_str = net_ip_mask.strNormal()
+                n["network"] = net_str
+            except Exception,ex:
+                pass
+
 
     def __populateDictionary(self):
         """ retrieve environment information for the
@@ -355,8 +355,24 @@ class NodeEnvHelper:
                           for v in memory_values ]
 
         network_values = self.node_proxy.exec_cmd( \
-               'ifconfig | awk \'{print $1, $2, $3}\' ;'
-               )[0].split("\n  \n")
+               'ifconfig | awk \'{print $1, $2, $3, $4}\' ;'
+               )[0].split("\n   \n")
+
+        # Process networks # TODO : rewrite this.
+        raw_network_list = network_values
+        network_list = []
+        if raw_network_list[0].find('not found') == -1:
+            for i in range(len(raw_network_list)):
+                split_item = raw_network_list[i].split("UP")
+                if cmp('',split_item[0]) !=0 :
+                    split_item = split_item[0].split('\n')
+                    interface_name = split_item[0].split()[0]
+                    ip_addr = ''
+                    if cmp('',split_item[1]) !=0:
+                        for i in range(1, len(split_item)):
+                            ip_addr += split_item[i] + '\n'
+                    network_list.append(dict([(key_network_interface_name, interface_name), (key_network_ip, ip_addr)]))
+
 
         disk_values = self.node_proxy.exec_cmd( \
                 'df -kh -P | grep ^/dev | awk \'{print $1, $2, $6}\''\
@@ -376,12 +392,128 @@ class NodeEnvHelper:
           disk_list.append(dict((disk_attributes[x],disk_values_split[x]) \
             for x in range(len(disk_values_split))))
 
+        # os details
+        os_values = self.node_proxy.exec_cmd( \
+            'uname -r;uname -s; uname -m;'\
+                )[0].split('\n')[:-1]
+        i = 0
+        os_dict = {}
+        for name in [key_os_release,key_os_system,key_os_machine]:
+            try:
+                os_dict[name] = os_values[i]
+            except ValueError:
+                pass
+            i = i+1
+
+        # Augment the information gathered from setup script.
+        discovery_file = '/var/cache/convirt/server_info'
+        alt_discovery_file = '/var/cache/convirt/server_info'
+        if not self.node_proxy.file_exists(discovery_file):
+            if self.node_proxy.file_exists(alt_discovery_file):
+                discovery_file = alt_discovery_file
+
+        discovered_info = None
+        if self.node_proxy.file_exists(discovery_file):
+            discovered_info = PyConfig(self, discovery_file)
+        
+        def_bridge = ""
+        if discovered_info:
+            os_dict[key_os_distro] = discovered_info["DISTRO"]
+            os_dict[key_os_distro_ver] = discovered_info["VER"]
+            os_dict[key_os_distro_string] = discovered_info["DISTRO"] + " " + \
+                discovered_info["VER"]
+            def_bridge = discovered_info["DEFAULT_BRIDGE"]
+
+
+        # Add some more network specific info.
+        cmd = "ls -ld /sys/class/net/*/device | sed -e 's/.*\/sys\/class\/net\///' -e 's/\/device.*//'"
+    
+        nics = {}
+        (output,exit_code) = self.node_proxy.exec_cmd( cmd)
+        if output and not exit_code:
+            nic_list = output.split('\n')
+            for nic in nic_list:
+                if nic:
+                    nics[nic] = { "name":nic }
+
+        bridges = {}
+        cmd = "ls -ld /sys/class/net/*/{bridge,brif/*} | sed -e 's/.*\/sys\/class\/net\///' -e 's/\/bridge.*//' -e 's/\->.*//'"
+        (output,exit_code) = self.node_proxy.exec_cmd( cmd)
+        if output and not exit_code:
+            base_entries = output.split('\n')
+            entries=[]
+            for e in base_entries:
+                if e and e.find('cannot access') < 0:
+                    entries.append(e)
+            # Populate bridges
+            for e in entries:
+                if e and e.find("/brif/") == -1:
+                    bridges[e] = { "name":e }
+            # Populate interfaces for bridges.
+            for e in entries:
+                if e.find("/brif/") > -1:
+                    (br_name,brif,ifname) = e.split("/")
+                    if ifname:
+                        ifname = ifname.strip()
+                    if ifname and (ifname.find("vif") == 0 or \
+                            ifname.find("tap")==0):
+                        # ignore virtual interface conventions.
+                        continue
+                    bridge = bridges[br_name]
+                    if bridge :
+                        interfaces = bridge.get("interfaces")
+                        if interfaces is not None:
+                            inteterfaces.append(ifname)
+                        else:
+                            bridge["interfaces"] = [ifname,]
+
+        # from the network info, get ip address and netmask and 
+        # construct net details from it.
+        # NOTE : There must be a better way of doing this.
+        # Look in to ioctl 
+        ipv4_re = re.compile("(.*):(.*) (.*):(.*)")
+        ipv4_re1 = re.compile("(.*):(.*) (.*):(.*) (.*):(.*)")
+        for n_info in network_list:
+            ifname = n_info[key_network_interface_name]
+            b = bridges.get(ifname)
+            n = nics.get(ifname)
+            if b or n:
+                # we need to process
+                ip_info = n_info[key_network_ip]
+                if ip_info:
+                    ip_list = ip_info.split('\n')
+                    for ip_entry in ip_list:
+                        m = ipv4_re.match(ip_entry)
+                        if not m:
+                            ipv4_re1.match(ip_entry)
+                        if m :
+                            l = len(m.groups())
+                            if l > 0 and  m.group(1).find("inet addr") == 0:
+                                if b: b["ip"] = m.group(2)
+                                if n: n["ip"] = m.group(2)
+                            if l > 2 and m.group(3).find("Mask") == 0:
+                                if b: b["netmask"] = m.group(4)
+                                if n: n["netmask"] = m.group(4)
+                            if l > 4 and m.group(5).find("Mask") == 0:
+                                if b: b["netmask"] = m.group(6)
+                                if n: n["netmask"] = m.group(6)
+                                
+                                
+        # find the network by using ip and netmask
+        for n in nics.itervalues():
+            self.populate_network(n)
+        for b in bridges.itervalues():
+            self.populate_network(b)
+
+        #import pprint; pprint.pprint(bridges)
         self.__dict = dict([('cpu_info',cpu_dict),
                             ('memory_info', memory_dict),
                             ('disk_info', disk_list),
-                            ('network_info',network_values),
-                            ('os_info', {}) ])
-
+                            ('network_info',network_list),
+                            ('nic_info',nics),
+                            ('bridge_info',bridges),
+                            ('os_info', os_dict),
+                            ('default_bridge', def_bridge) ])
 
 
 
