@@ -178,6 +178,14 @@ class VM:
     def setMem(self,value):
        self.node.get_vmm().setMem(self.id, value)
 
+    # attach disks in memory
+    def attachDisks(self,attach_disk_list):
+       self.node.get_vmm().attachDisks(self.id,attach_disk_list)
+
+    # detach disks in memory
+    def detachDisks(self,detach_disk_list):
+       self.node.get_vmm().detachDisks(self.id,detach_disk_list)
+
     # Associate a config
     def get_config(self):
        return self._config 
@@ -268,6 +276,7 @@ class DiskEntry:
 class ImageDiskEntry(DiskEntry):
    CREATE_DISK = "CREATE_DISK"
    USE_DEVICE = "USE_DEVICE"
+   USE_ISO = "USE_ISO"
    USE_REF_DISK = "USE_REF_DISK"
    
    def __init__(self, input, image_conf):
@@ -295,6 +304,10 @@ class ImageDiskEntry(DiskEntry):
       device = self.device
 
       # Vars
+      pos = device.find(":cdrom")
+      if pos > -1:
+         device=device[0:pos]
+
       create_var = device + "_disk_create"
       image_src_var = device + "_image_src"
       image_src_type_var = device + "_image_src_type"
@@ -306,19 +319,25 @@ class ImageDiskEntry(DiskEntry):
       self.option = self.CREATE_DISK
       self.disk_create = self.image_conf[create_var]
       self.disk_type = self.image_conf[disk_type_var]
+      self.size = self.image_conf[size_var]
+
       if not self.disk_type :
          if self.type == "file":
             self.disk_type = "VBD"
          if self.type == "phy": # assume physical device e.g. cdrom
             self.disk_type = ""
+
       
       if self.image_conf[image_src_var] :
          self.option = self.USE_REF_DISK
-         
       elif self.type == "phy" :
          self.option = self.USE_DEVICE
+      elif self.type=="file" :
+         if not self.size and self.filename != "":
+            self.option = self.USE_ISO
+            self.disk_type = "ISO"
+
 
-      self.size = self.image_conf[size_var]
       if self.option == self.CREATE_DISK or self.option == self.USE_REF_DISK:
          if not self.size:
             self.size = 10000
@@ -332,7 +351,41 @@ class ImageDiskEntry(DiskEntry):
       self.fs_type = self.image_conf[disk_fs_type_var]
 
 
+class vifEntry:
+   def __init__(self, vif_str, image_config=None):
+      self.vif_map = {}
+      if vif_str:
+         self.vif_map= dict([[z.strip() for z in y.split("=")] 
+                             for y in [x.strip() 
+                                       for x in vif_str.split(",")]])
+   def get_mac(self):
+      return self.vif_map.get("mac")
 
+   def set_mac(self, v):
+      self.vif_map["mac"] = v
+      
+
+   def get_bridge(self):
+      return self.vif_map.get("bridge")
+
+   def set_bridge(self,v):
+      self.vif_map["bridge"] = v
+
+   def __repr__(self):
+      s = ''
+      def_keys = ["mac", "bridge"]
+      keys = self.vif_map.keys()
+      for k in def_keys:
+         if k in keys:
+            keys.remove(k)
+
+      for k in def_keys + keys:
+         v=self.vif_map.get(k)
+         if v is not None:
+            if s != '':
+               s = s + ","
+            s = s + "%s=%s" %(k,v)
+      return s
 
 
 # Network class
@@ -355,7 +408,7 @@ class VMStorageStat:
    FILE = "FILE"
    UNKNOWN = "UKNOWN"
 
-   GB_BYTES = 1024 * 1024 * 1024
+   GB_BYTES = 1024.0 * 1024.0 * 1024.0
    
    def __init__(self, config):
       self.config = config
@@ -501,17 +554,19 @@ class VMStorageStat:
           if filename.find("/dev") == 0:
              # assume block device
              dev_type = self.BLOCK
-             fd = None
              try:
                 try:
-                   fd = self.config.managed_node.node_proxy.open(filename, "r")
-                   fd.seek(0, os.SEEK_END)
-                   size = fd.tell()
+                   cmd="""python -c "import os; filename='%s'; fd=open(filename,'r'); fd.seek(0,2); size=fd.tell() ; fd.close(); print size" """ % filename
+                   (output, code) = self.config.managed_node.node_proxy.exec_cmd(cmd)
+                   #print "cmd ", cmd, " code -> " , code, " out-> ", output
+                   if code == 0:
+                      size=eval(output)
+                   else:
+                      raise Exception(output)
                 except Exception, ex:
                    print "error getting disk size for ", filename, ex
              finally:
-                if fd:
-                   fd.close()
+                pass
           else:
              if self.config.managed_node.is_remote():
                 size = f_stat.st_size
@@ -610,6 +665,15 @@ class VMConfig(PyConfig):
             reslist.append(ImageDiskEntry(str, image_config))
         return reslist
 
+    def getNetworks(self, image_config=None):
+        """
+        This method returns a more structured version of the config file's
+        vif entry
+        """
+        reslist = []
+        for str in self["vif"]:
+            reslist.append(vifEntry(str, image_config))
+        return reslist
 
     # Assumes that the file read contains single line values
     # now preserves the comments and order
