@@ -188,11 +188,11 @@ class VNode(ManagedNode):
 
     # directory where the config files should be stored
     # for them to come up when machine boots
-    def get_auto_config_dir():
+    def get_auto_config_dir(self):
         return ""
     # directoy to store vm configs.
-    def get_config_dir():
-        return ""
+    def get_config_dir(self):
+        return "/var/cache/convirt/vm_configs"
     
     def get_platform(self):
         return self.platform
@@ -280,11 +280,10 @@ class VNode(ManagedNode):
         self.refresh()
 
     def disconnect(self):
-        ManagedNode.disconnect(self)
-        if self.get_vmm() is not None:
+        if self._vmm is not None:
             self.get_vmm().disconnect()
             self._vmm = None
-
+        ManagedNode.disconnect(self)
 
     # Factory and helpers to create VMs
     # Given a filename, return concrete config object
@@ -670,8 +669,11 @@ class DomListHelper:
             #Add to managed_domfiles the auto_list files
             new_auto_node = False
             for filename in auto_list:
-                if filename.startswith("."): # for Gentoo
+                if filename.startswith(".") : # for Gentoo
                     continue
+                if filename.endswith(".xml") : # skip xml
+                    continue
+
                 auto_file_name = auto_dir + '/' + filename
                 try:
                     i = self.node.managed_domfiles.index(auto_file_name)
