@@ -74,6 +74,7 @@ class KVMProxy(VMM):
     kvm_options = kvm_options_with_v + kvm_options_no_v
     qemu_options = kvm_options 
 
+
     # share the proxy operations and ssh_transport
     def __init__(self, node):
         self.node = node
@@ -82,9 +83,14 @@ class KVMProxy(VMM):
         self.channel_map = {}
         self.vm_list = None
         self._info = None
+        self._slashes = None
+
+        if self.node_proxy.file_exists("/usr/bin/qemu-kvm"):
+#            print "Switching kvm binary to 'qemu-kvm'"
+            self.kvm_binary = "qemu-kvm"
 
         if self.node_proxy.file_exists("/usr/bin/kvm"):
-            print "Switching kvm binary to 'kvm'"
+#            print "Switching kvm binary to 'kvm'"
             self.kvm_binary = "kvm"
 
 
@@ -153,6 +159,8 @@ class KVMProxy(VMM):
         (code, output) = self.read_till_prompt(id)
         if code != 0 :
             print id, code, output
+            if not code:
+                code = 127
             raise Exception("Error,connecting to the VM %s, error code = %d msg = %s" % (id, code, output))
 
         ## check if chan established... and working.. or socat
@@ -198,7 +206,7 @@ class KVMProxy(VMM):
                            ((e.args[0] == errno.EAGAIN or e.args[0] == 4)):
                         try:
                             select.select([],[],[],.1)
-                        except ex:
+                        except Exception, ex:
                             print ex
                             pass
                         continue
@@ -316,13 +324,22 @@ class KVMProxy(VMM):
                 info["name"] = m[m.rfind('/')+1:]
 
 
+    def get_slashes(self):
+        if self._slashes is None:
+            ver_num = self.get_version_int(self.info().get(key_version))
+            if ver_num > 77:
+                self._slashes = ""
+            else:
+                self._slashes =  "//"
+        return self._slashes
+
     ################ Implement some of the VMM interface #############
     # Start, Stop, Pause, Resume, Kill, Take snapshot, Restore Snapshot
 
     """ info about the vmm capabilities and node info as seen by vmm """
     def info(self):
         if self._info is None:
-            info_cmd = self.kvm_binary + '| head -1'
+            info_cmd = self.kvm_binary + ' -help | head -1'
             (out, code) = self.node_proxy.exec_cmd(info_cmd,
                                                    self.kvm_binary_path)
             if code == 0:
@@ -425,9 +442,22 @@ class KVMProxy(VMM):
         self.vm_list = vms
         return vms
 
+    def get_version_int(self, v_str):
+        v= v_str
+        if not v:
+            return 0
+        index=v.find('kvm-')
+        s_index = index + 4
+        if s_index > -1:
+            index=s_index
+            while index < len(v) and v[index].isdigit() : 
+                index = index + 1
+            return int(v[s_index:index])
+        return 0
+                
 
     # Use the config and start a new vm.
-    def start(self,config):
+    def start(self,config,timeout=5):
         if config is None:
             raise Exception("No context provided to start the vm")
 
@@ -436,9 +466,10 @@ class KVMProxy(VMM):
         info = self.info()
         if info.get(key_version):
             v = info.get(key_version)
-            if v.find('(kvm-') < 0:
+            if v.find('kvm-') < 0:
                 raise Exception("You seem to have an older version of KVM/QEMU\n The version does not contain 'kvm-' token :%s\n Please make sure kvm-70 or higher is installed and is in PATH." % v)
-        
+
+
         # take the config.. and generate a cmdline for kvm/qemu
         cmd = self.kvm_binary
         known_options = self.kvm_options
@@ -499,24 +530,17 @@ class KVMProxy(VMM):
                     vlan=-1
                     for vif in vifs:
                         vlan = vlan + 1
-                        parts = vif.split(', ')
-                        x = dict([p.split('=') for p in parts])
+                        parts = vif.split(',')
+                        x = dict([p.strip().split('=') for p in parts])
                         macaddr = x.get("mac")
                         if not macaddr:
                             macaddr=utils.randomMAC()
                         opt_val = "nic,vlan=%d,macaddr=%s" % ( vlan, macaddr)
-                        # interface name
-                        ifname = x.get("ifname")
-                        if ifname:
-                            opt_val = opt_val + ",ifname=" + ifname
+                        
                         # model
                         model = x.get("model")
                         if model:
                             opt_val = opt_val + ",model=" + model
-                        # script
-                        script = x.get("script")
-                        if script:
-                            opt_val = opt_val + ",script=" + script
                             
                         cmdline = self.process_option(cmdline, "net",
                                                       opt_val, known_options)
@@ -538,6 +562,35 @@ class KVMProxy(VMM):
                                 mode = "user"
 
                         opt_val = "%s,vlan=%d" % (mode, vlan)
+
+                        if mode == "tap":
+                            # interface name
+                            ifname = x.get("ifname")
+                            if ifname:
+                                opt_val = opt_val + ",ifname=" + ifname
+                            # script
+                            script = x.get("script")
+                            if script:
+                                opt_val = opt_val + ",script=" + script
+                            else:
+                                # see if the bridge specific script is there.
+                                if bridge:
+                                    s1 = "/etc/kvm/kvm-ifup-%s" % (bridge,)
+                                    s2 = "/etc/kvm/qemu-ifup-%s" % (bridge,)
+                                    s3 = "/etc/qemu-ifup-%s" % (bridge,)
+                                    s4 = "/etc/qemu/qemu-ifup-%s" % (bridge,)
+                                    for s in [ s1, s2, s3, s4 ]:
+                                        if self.node_proxy.file_exists(s):
+                                            # assume it is executable.
+                                            opt_val = opt_val + ",script=" + s
+                                            break
+
+                        elif mode == "user":
+                            # hostname
+                            hname = x.get("hostname")
+                            if hname:
+                                opt_val = opt_val + ",hostname=" + hname
+
                         cmdline = self.process_option(cmdline, "net",
                                                        opt_val, known_options)
                         
@@ -581,11 +634,10 @@ class KVMProxy(VMM):
         if not self.node_proxy.file_exists(self.monitor_dir):
             mkdir2(self.node, self.monitor_dir)
         monitor_path = os.path.join(self.monitor_dir, config.get("name"))
-        cmdline = cmdline + " -monitor unix:%s" % monitor_path + \
-                  ",server,nowait"
-
-        cmdline = cmdline + " -pidfile  " + os.path.join(self.pid_dir,
-                                                         config.get("name"))
+        cmdline = cmdline + " -monitor " + """"unix:%s,server,nowait\"""" % (monitor_path,)
+                 
+        pid_fname = os.path.join(self.pid_dir, config.get("name"))
+        cmdline = cmdline + " -pidfile  " + """"%s\"""" % (pid_fname,)
                     
         # daemonize.. the command can return
         cmdline = cmdline + " -daemonize"
@@ -597,11 +649,13 @@ class KVMProxy(VMM):
 
         print "CMDLINE ***** ", cmdline
         (output, ret) = self.node_proxy.exec_cmd(cmdline, self.kvm_binary_path,
-                                                 5)
+                                                 timeout)
         if ret != 0:
             print "start failed :", cmdline,output
             raise Exception((output, ret))
+        print "start : success ", output
         self.get_vms()
+        return config.get("name")
 
 
     def process_option(self, cmdline, opt, value, known_options):
@@ -617,12 +671,12 @@ class KVMProxy(VMM):
                 elif isinstance(value, list):
                     for v in value:
                         cmdline = cmdline + " -" + opt
-                        cmdline = cmdline + " " + v
+                        cmdline = cmdline + " " + """%s\"""" % (v,)
                     return cmdline
 
                 if isinstance(value, str):
                     cmdline = cmdline + " -" + opt
-                    cmdline = cmdline + " " + value
+                    cmdline = cmdline + " " + """"%s\"""" % (value,)
                 else:
                     print "ERROR : Unknown type, Ignoring ", opt, value
                     
@@ -673,11 +727,18 @@ class KVMProxy(VMM):
         if prompt and output == cmd:
             # vm stopped now lets use migrate to save it to the file
             # save vmconfig context ?
-            cmd = "migrate " + "file://" + filename 
+            ver_num=self.get_version_int(self.info().get(key_version))
+            if ver_num > 77:
+                cmd = "migrate " + """"exec: gzip -c > %s.gz\"""" % (filename,)
+            else:
+                cmd = "migrate " + "file://%s" % (filename,)
             (output,prompt) = self.send_command(id, cmd)
             if prompt and output == cmd:
                 print "vm state saved to " + filename
-                ctx_filename = filename + ".ctx"
+                if ver_num > 77:
+                    ctx_filename = filename + ".gz.ctx"
+                else:
+                    ctx_filename = filename + ".ctx"
                 contents = cfg.get_contents()
                 outfile = self.node.node_proxy.open(ctx_filename, 'w')
                 outfile.write(contents)
@@ -686,6 +747,7 @@ class KVMProxy(VMM):
                 self.destroy(id)
                 return True
             else:
+                #print """'%s' '%s'""" % (output, cmd)
                 raise Exception(cmd + ":[" + output + "]")
         else:
             raise Exception(cmd + ":[" + output + "]")
@@ -700,8 +762,21 @@ class KVMProxy(VMM):
                 filename = filename[0:-4]
         
         cfg = self.node.new_config(ctx_file)
-        cfg["incoming"] = "file://" + filename
-        self.start(cfg)
+        ver_num=self.get_version_int(self.info().get(key_version))
+        if ver_num > 77:        
+            cfg["incoming"] = "exec: gzip -c -d " + filename 
+        else:
+            cfg["incoming"] = "file://" + filename 
+        id = self.start(cfg,120)
+        #self.unpause(id) 
+        # inline for now
+        (output,prompt) = self.send_command(id,"cont")
+        if prompt and output == "cont":
+            return True
+        else:
+            raise Exception("restore/resume:" + output)
+
+ 
 
     def quit_vm(self,id):
         cmd = "quit"
@@ -724,8 +799,8 @@ class KVMProxy(VMM):
         # kill the process.
         vm = self.vm_list.get(id)
         pid = vm["pid"]
-        (output, code) = self.node_proxy.exec_cmd("kill -9 " + pid)
-        if code != 0 :
+        (output, code) = self.node_proxy.exec_cmd("kill -9 " + pid, timeout=5)
+        if code != 0 and code != -99:
             raise Exception("Error : VM Kill %s %s: %s : %s" % (id, pid, str(code), output))
 
 
@@ -740,7 +815,7 @@ class KVMProxy(VMM):
         #        listen on all interfaces.
 
         print "LIVE MiGRATION ", live
-        cfg["incoming"] = "tcp://0:" + str(port)
+        cfg["incoming"] = "tcp:%s0:%s" % (self.get_slashes(), str(port))
 
         try:
             dst.create_dom(cfg)
@@ -763,7 +838,9 @@ class KVMProxy(VMM):
                 else:
                     raise Exception("error stopping VM during migration :" + output)
             
-            cmd = "migrate tcp://" + dst.get_address() +":"+str(port)
+            cmd = "migrate tcp:%s%s:%s" % (self.get_slashes(), 
+                                           dst.get_address(),
+                                           str(port))
             print "Initiating migration ", cmd
             (output,prompt) = self.send_command(id,cmd)
             if prompt and output == cmd:
@@ -807,7 +884,13 @@ class KVMProxy(VMM):
     def setMem(self, id, value):
         raise Exception("KVM : Can not change memory for running vm.")
 
+   # attach disks in memory
+    def attachDisks(self,id,attach_disk_list):
+       raise Exception("KVM : Can not attach disks for running vm.")
 
+    # detach disks in memory
+    def detachDisks(self,id,detach_disk_list):
+       raise Exception("KVM : Can not detach disks for running vm.")
 
 if __name__ == '__main__':
     print "Enter password :"
@@ -825,7 +908,7 @@ if __name__ == '__main__':
     config["hda"]="/vm_disks/fc-7.qcow2"
     config["m"] = 512
     config["name"] = "fc-7"
-    #config["monitor"] = "unix:/tmp/" + config["name"]
+    #config["monitor"] = "unix:/var/cache/convirt/monitors/" + config["name"]
     config["vnc"] = ":5"
 
     import pprint
