@@ -77,8 +77,8 @@ class XenNode(VNode):
     def get_auto_config_dir(self):
         return '/etc/xen/auto'
 
-    def get_config_dir(self):
-        return '/etc/xen'
+    #def get_config_dir(self):
+    #    return '/etc/xen'
 
 
         
@@ -103,7 +103,9 @@ class XenNode(VNode):
             #else
             #ALLOW Dom 0 to be in the list too
             #This would allow twaking mem and cpus
-            current_dict[vm.name] = vm
+            # The xend-managed returns non-running vms too. ignore id = None 
+            if vm.id is not None: 
+                current_dict[vm.name] = vm
             
         return current_dict
 
@@ -150,6 +152,8 @@ class XenNode(VNode):
 
     def is_image_compatible(self, image):
         if image:
+            if image.is_hvm() and self.is_hvm():
+		return True
             if self.get_platform() == image.get_platform() :
                 # xen image check for hvm
                 if image.is_hvm():
@@ -260,7 +264,7 @@ class XenNode(VNode):
                 err_list.append(("Ramdisk", "Ramdisk %s for %s vm not found on destination node." % (ramdisk.strip(), vm.name)))
 
        # hvm availablity
-       if vm_conf.is_hvm() \
+       if vm_conf and vm_conf.is_hvm() \
           and not self["xen_caps"].find("hvm"):
           err_list.append(("HVM","VM %s requires hvm capabilities which are not found on destination node." % (vm.name)))
 
@@ -499,7 +503,7 @@ if __name__ == "__main__":
     m = { 'VM_NAME':'foo', 'IMAGE_NAME':'anaconda' } 
     dom_config.instantiate_config(m)
     print dom_config.default_computed_options, dom_config.get_computed_options()
-    dom_config.save("/tmp/test_config")
+    dom_config.save("/foo/test_config")
 
     sys.exit(0)
     
@@ -578,21 +582,21 @@ if __name__ == "__main__":
     newcfg = XenConfig(managed_node)
     newcfg["name"] = "Txx"
     newcfg["memory"] = 299
-    newcfg.set_filename("/tmp/Txx")
+    newcfg.set_filename("/foo/Txx")
     newcfg.write()
 
-    f = managed_node.node_proxy.open("/tmp/Txx")
+    f = managed_node.node_proxy.open("/foo/Txx")
     x = f.read(1024)
     print x
     f.close()
 
-    print "### read config from /etc/xen/auto and write them to /tmp"
+    print "### read config from /etc/xen/auto and write them to /foo"
     ## Dom Config
     for f in managed_node.node_proxy.listdir("/etc/xen/auto"):
         fin = "/etc/xen/auto/"+f
         print fin
         d = XenConfig(managed_node, fin)
-        d.save("/tmp/" + f)
+        d.save("/foo/" + f)
     
 
     print "### get first file in /etc/xen/auto and dump its info"
