@@ -18,7 +18,7 @@
 #
 
 import ConfigParser, subprocess, platform
-import sys, os, os.path, socket, types, tempfile, re, glob, md5
+import sys, os, os.path, socket, types, tempfile, re, glob
 
 import shutil, urllib,urllib2,urlparse
 import convirt.core.utils.constants
@@ -32,12 +32,24 @@ import xml.dom.minidom
 from xml.dom.minidom import Node
 import webbrowser
 
+
+try:
+    import hashlib
+    md5_constructor=hashlib.md5
+except ImportError,e:
+    import md5
+    md5_constructor=md5.new
+
+
 # ease of use
 constants = convirt.core.utils.constants
 
 class dynamic_map(dict):
-    def __init__(self):
-        dict.__init__(self)
+    def __init__(self, another_dict=None):
+        if another_dict:
+            dict.__init__(self, another_dict)
+        else:
+            dict.__init__(self)
 
     def __getattr__(self, name):
         if self.has_key(name):
@@ -52,6 +64,24 @@ class dynamic_map(dict):
 
     def __setattr__(self,name,value):
         self[name] = value
+
+# allows values to be functions or method and eval them 
+# useful for generating multiple nic addresses in template substitution.
+class fv_map(dict):
+    def __init__(self):
+        dict.__init__(self)
+
+    def __getitem__(self, name):
+        if dict.has_key(self,name):
+            value=dict.__getitem__(self, name)
+            if isinstance(value, types.MethodType) or \
+               isinstance(value, types.FunctionType) :
+                value = value() # evaluate it
+                return value
+
+        return dict.__getitem__(self, name)
+
+
         
 class XMConfig(ConfigParser.SafeConfigParser):
     """ ConVirt's configuration management class. """
@@ -194,11 +224,9 @@ class XMConfig(ConfigParser.SafeConfigParser):
         a_store = os.path.join(base, 'appliance_store')
         updates_file = os.path.join(base, 'updates.xml')
         
-        self.set(self.PATHS, constants.prop_disks_dir, '')
-        self.set(self.PATHS, constants.prop_snapshots_dir, '')
+        self.set(self.PATHS, constants.prop_snapshots_dir, "/var/cache/convirt/vm_snapshots")
         self.set(self.PATHS, constants.prop_snapshot_file_ext, '.snapshot.xm')
-        self.set(self.PATHS, constants.prop_xenconf_dir,'/etc/xen')
-        self.set(self.PATHS, constants.prop_cache_dir, '/var/cache/convirt')
+        self.set(self.PATHS, constants.prop_cache_dir, "/var/cache/convirt")
         self.set(self.PATHS, constants.prop_exec_path, '$PATH:/usr/sbin')
         self.set(self.PATHS, constants.prop_image_store, i_store)
         self.set(self.PATHS, constants.prop_appliance_store, a_store)
@@ -223,8 +251,8 @@ class XMConfig(ConfigParser.SafeConfigParser):
         self.set(self.CLIENT_CONFIG,
                      constants.prop_chk_updates_on_startup,'True')
 
-        self.set(self.CLIENT_CONFIG,
-                     constants.prop_vncviewer_options,'-PreferredEncoding=hextile')
+        #self.set(self.CLIENT_CONFIG,
+        #             constants.prop_vncviewer_options,'-PreferredEncoding=hextile')
 
         
         #self.set(self.CLIENT_CONFIG, constants.prop_browser, '/usr/bin/yelp')
@@ -252,6 +280,7 @@ class XMConfig(ConfigParser.SafeConfigParser):
         fp.close()
         for s in self.std_sections:
             if not temp.has_section(s):
+                print "section " + s + " not found"
                 return False
         return True
     
@@ -751,6 +780,7 @@ from threading import Thread
 class Worker(Thread):
     def __init__(self, fn, succ, fail,progress=None):
         Thread.__init__(self)
+        self.setDaemon(True)
         self.fn = fn
         self.succ = succ
         self.progress = progress
@@ -772,7 +802,7 @@ class Worker(Thread):
 # can be used by UI to display the updates.
 class UpdatesMgr:
     update_url = "http://www.convirture.com/updates/updates.xml"
-    updates_file = "/tmp/updates.xml"
+    updates_file = "/var/cache/convirt/updates.xml"
     
     def __init__(self, config):
         self.config = config
@@ -786,20 +816,25 @@ class UpdatesMgr:
         if not self.updates_file:
             self.updates_file = UpdatesMgr.updates_file
 
-        # file is not writable..lets create a tmp file
-        if not os.access(self.updates_file,os.W_OK):
-            (t_handle, t_name) = tempfile.mkstemp(prefix="updates.xml",
-                                                          dir="/tmp")
-            self.updates_file = t_name
-            os.close(t_handle) # Use the name, close the handle.
 
     def fetch_updates(self):
         update_items = []
 
         try:
+            # file is not writable..lets create a tmp file
+            if not os.access(self.updates_file,os.W_OK):
+                (t_handle, t_name) = tempfile.mkstemp(prefix="updates.xml")
+                self.updates_file = t_name
+                os.close(t_handle) # Use the name, close the handle.
+                
             fetch_isp(self.url, self.updates_file, "/xml")
         except Exception, ex:
             print "Error fetching updates ", ex
+            try:
+                if os.path.exists(self.updates_file):
+                    os.remove(self.updates_file)
+            except:
+                pass
             return update_items
 
 
@@ -815,6 +850,13 @@ class UpdatesMgr:
 
                 update_items.append(info)
 
+        # cleanup the file
+        try:
+            if os.path.exists(self.updates_file):
+                os.remove(self.updates_file)
+        except:
+            pass
+        
         return update_items
 
 
@@ -947,7 +989,7 @@ def copyToRemote(src,dest_node,dest_dir, dest_name = None, hashexcludes=[]):
 def generateHashFile(filename):
 	f = file(filename,'rb')
 	fw = file(filename + ".hash", 'wb')
-	m = md5.new()
+	m = md5_constructor()
  
 	readBytes = 1024 # read 1024 bytes per time
 	try:
@@ -972,7 +1014,7 @@ def generateHashFile(filename):
 # If not generate a new hash value and add modification time.
 def updateHashFile(filename):
     fhash = file(filename + ".hash", 'rb')
-    m = md5.new()
+    m = md5_constructor()
 
     try:
         readHash = fhash.read()
@@ -1066,7 +1108,7 @@ def search_tree(tree, key):
     for elem in l:
         #print "processing ..", elem[0], key
         if type(elem) is list:
-            if elem[0] == key:
+            if len(elem) > 0 and elem[0] == key:
                 if len(elem) > 2:
                     #print "returning [[v1],[v2],..] from NV"
                     return elem[1:]
@@ -1421,7 +1463,277 @@ def get_prop(map, key, default=None):
         return map[key]
     else:
         return default
-    
+
+
+
+def cleanupQCDomain(managed_node, name, vbdlocation=''):
+    """ Delete the xen configuration file and associated
+    disk devices created during a quickcreate"""
+
+    if name in managed_node.get_dom_names():
+        dom = managed_node.get_dom(name)
+        dom_config = None
+        if dom:
+            dom_config = dom.get_config()
+
+        if dom_config:
+            domfilename = dom_config.filename
+            if domfilename and managed_node.node_proxy.file_exists(domfilename):
+                managed_node.node_proxy.remove(domfilename)
+
+
+            for file in dom_config.getDisks():
+                # skip read only volumes, they are likely to be shared
+                # or cdroms.
+                # THIS IS NOT COMPLETE SOLN, NEED to prompt user for each
+                # disk is the right soln
+                if file.mode.find("w") == -1:
+                    continue
+                # dont mess with /dev/disk/ area. (Lun, AOE disks)
+                if file.filename and (file.filename.strip().find("/dev/disk/")==0 or file.filename.strip().find("/dev/etherd/") == 0):
+                    continue
+                if file.type == 'phy':
+                    try:
+                        managed_node.lvm_proxy.removeLogicalVolume(file.filename)
+                        print 'deleting: ' + file.filename
+                    except OSError, err:
+                        print "error deleting " + file.filename,err
+                elif file.type in ['file','tap:aio','tap:qcow','tap:vmdk']:
+                    if managed_node.node_proxy.file_exists(file.filename):
+                        print 'deleting: ' + file.filename
+                        managed_node.node_proxy.remove(file.filename)
+    else:
+        print "Couldn't find the domain %s. Skipping deletion" % name
+
+# Provisioning through cli
+def provision_vm(image_store,image_name,eGroup,managed_node,storage_manager,vname,memory,vcpus):
+
+
+    ctx = dynamic_map()
+    ctx.image_store = image_store
+    image_id=""
+    images=image_store.get_images()
+    for id in images.keys():
+        img=images[id]
+        if image_name==img.name:
+            image_id=img.id
+            image=img
+            break
+
+    if image_id=="":
+        raise e
+
+    ctx.image_id=image_id
+    ctx.managed_node= managed_node
+    ctx.vm = None
+    ctx.server_pool= eGroup
+    ctx.platform = managed_node.get_platform()
+    ctx.storage_manager = storage_manager
+    vm_config,image_config=image.get_configs()
+
+    if ctx.server_pool is not None:
+            grp_settings = ctx.server_pool.getGroupVars()
+            merge_pool_settings(vm_config,
+                                      image_config,
+                                      grp_settings, True)
+
+
+    vm_config['name']=vname
+    if memory is not None:
+        vm_config['memory']=memory
+    if vcpus is not None:
+       vm_config['vcpus']=vcpus
+
+    vm_config_path=os.path.join('$VM_CONF_DIR',vname)
+    vm_config.set_filename(vm_config_path)
+
+    ctx.vm_config=vm_config
+    ctx.image_config=image_config
+
+    kick_off_cli_provisioning(ctx,image_name)
+
+# Provisioning process
+def kick_off_cli_provisioning(context,img_name):
+
+    # instantiate the configs
+    v_config = context.vm_config
+    i_config = context.image_config
+    # add/set a uui to the config
+    v_config["uuid"] = uuidToString(randomUUID())
+
+    # create is_remote map by position.
+    image_id=context.image_id
+    try:
+
+        store,managed_node,vm_config_file=vm_config_write(context,image_id,v_config,
+                                            i_config,img_name)
+         # execute the provisioning script
+        (out, exit_code, log_filename)=store.execute_provisioning_script(
+                                     managed_node,
+                                     image_id,
+                                     v_config,
+                                     i_config)
+
+#        print "out==",out, "exit_code=",exit_code,"log", log_filename
+        if exit_code == 0:
+            try:
+                managed_node.add_dom_config(vm_config_file)
+            except Exception, str:
+                raise Exception(str)
+
+
+    except Exception, str:
+        raise Exception(str)
+
+
+# Config file writing
+def vm_config_write(context,image_id,v_config,i_config,img_name):
+            # update is_remote and change protocol
+
+        try:
+            is_remote_list = []
+            s_stats = v_config.get_storage_stats()
+            for de in v_config.getDisks():
+                r = s_stats.get_remote(de.filename)
+                is_remote_list.append(r)
+
+            img_location = context.image_store.get_image(image_id).location
+            if img_location:
+                img_location = os.path.basename(img_location)
+
+            instantiate_configs(context.managed_node,
+                                     context.image_store,
+                                     img_name,
+                                     img_location,
+                                     v_config,
+                                     i_config)
+
+
+            store = context.image_store
+            managed_node = context.managed_node
+
+            vm_config_file  = v_config.filename
+
+
+            v_config["image_name"] = img_name
+            v_config["image_id"] = image_id
+            v_config["platform"] = managed_node.get_platform()
+            # see if the following can be moved to execute_prov script
+            v_config.set_managed_node(managed_node)
+            change_file_2_tap = False
+            is_hvm_image = context.image_store.get_image(image_id).is_hvm()
+            if v_config["platform"] == 'xen' and not is_hvm_image:
+                change_file_2_tap = True
+
+
+            changed = False
+            changed_disks = []
+
+            ndx = 0
+            ll = len(is_remote_list)
+            s_stats = v_config.get_storage_stats()
+            for de in v_config.getDisks():
+                if de and de.type == "file" and change_file_2_tap:
+                    de.type = "tap:aio"
+                    changed = True
+                changed_disks.append(repr(de))
+
+                r = None
+                if ndx < ll:
+                    r = is_remote_list[ndx]
+                s_stats.set_remote(de.filename,r)
+                ndx += 1
+
+
+            if changed: # update the disk entry
+                print "Updating disk to new value : for tap:aio", changed_disks
+                v_config["disk"] = changed_disks
+
+            v_config.write()
+
+            return store,managed_node,vm_config_file
+        except Exception, e:
+            raise e
+
+
+def instantiate_configs(managed_node,
+                            image_store, image_name, image_location,
+                            vm_config, image_config):
+
+    # get prepare template map
+    try:
+        template_map = fv_map()
+        store_location = image_store.get_remote_location(managed_node)
+
+        template_map["IMAGE_STORE"] = store_location
+        template_map["IMAGE_NAME"] = image_name
+        template_map["IMAGE_LOCATION"] = image_location
+        template_map["VM_NAME"] = vm_config["name"]
+
+        # default conventions
+        def_bridge="xenbr0"
+        if managed_node.platform == 'kvm':
+            def_bridge='br0'
+
+        # see if the managed_node has the default bridge
+        bridge = managed_node.get_default_bridge()
+        if bridge:
+            print "setting default bridge from discoverd information", bridge
+            def_bridge = bridge
+        elif image_config["DEFAULT_BRIDGE"] : #cheating
+            def_bridge = image_config["DEFAULT_BRIDGE"]
+
+
+        template_map["DEFAULT_BRIDGE"] = def_bridge
+
+        # Autogenerated MAC address
+        # TODO: keep track of generated MAC's
+        template_map["AUTOGEN_MAC"] = randomMAC # NOTE : passing function reference
+
+        image_config.instantiate_config(template_map)
+
+        # instantiate vm_config with image config
+        vm_config.instantiate_config(template_map)
+
+        if image_config is not None:
+            vm_config.instantiate_config(image_config)
+    except Exception, e:
+        raise e
+
+
+
+def merge_pool_settings(vm_config,
+                            image_config,
+                            pool_settings,
+                            append_missing=False):
+        gconfig = {}
+
+        for key in pool_settings:
+            gconfig[key] = pool_settings[key]
+
+        for key in vm_config:
+            if key in gconfig:
+                vm_config[key] = gconfig[key]
+                del gconfig[key]
+
+        for key in image_config:
+            if key in gconfig:
+                image_config[key] = gconfig[key]
+                del gconfig[key]
+
+        if append_missing is True:
+            for key in gconfig:
+                image_config[key] = gconfig[key]
+
+
+
+class dummyStream:
+	''' dummyStream behaves like a stream but does nothing. '''
+	def __init__(self): pass
+	def write(self,data): pass
+	def read(self,data): pass
+	def flush(self): pass
+	def close(self): pass
 #########################
 # SELF TEST
 #########################
@@ -1468,7 +1780,7 @@ if __name__ == '__main__':
     # XMConfig tests
     #    
 
-    TEST_CONFIGFILE = '/tmp/convirt.conf'
+    TEST_CONFIGFILE = '/foo/convirt.conf'
     
     print "\nXMConfig interface test STARTING\n"
     
